package cn.iocoder.yudao.module.system.controller.app.attendance;

import cn.hutool.json.JSON;
import cn.hutool.json.JSONUtil;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.module.system.controller.app.attendance.dto.AttendancePunchDTO;
import cn.iocoder.yudao.module.system.controller.app.attendance.dto.AttendancePunchPageDTO;
import cn.iocoder.yudao.module.system.controller.app.attendance.dto.AttendanceStatisticsByCycleDTO;
import cn.iocoder.yudao.module.system.controller.app.attendance.dto.AttendanceStatisticsByDayDTO;
import cn.iocoder.yudao.module.system.controller.app.attendance.vo.AttendancePunchPageVO;
import cn.iocoder.yudao.module.system.controller.app.attendance.vo.AttendancePunchVO;
import cn.iocoder.yudao.module.system.controller.app.attendance.vo.AttendanceStatisticsVO;
import cn.iocoder.yudao.module.system.dal.dataobject.attendance.punchrecord.AttendancePunchRecordDO;
import cn.iocoder.yudao.module.system.service.attendance.AttendanceService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.List;
import java.util.Map;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;

@Tag(name = "考勤 App - 考勤管理")
@RestController
@RequestMapping("/system/attendance")
@Validated
public class AttendanceController {
    @Resource
    private AttendanceService attendanceService;

    @GetMapping("/getPunchPage")
    @Operation(summary = "获取考勤页面")
    public CommonResult<AttendancePunchPageVO> getPunchPage(@ModelAttribute AttendancePunchPageDTO dto) {
        AttendancePunchPageVO vo = attendanceService.getPunchPage(dto);
        return success(vo);
    }

    @PostMapping("/punch")
    @Operation(summary = "打卡")
    public CommonResult<AttendancePunchVO> punch(@RequestBody AttendancePunchDTO dto) {
        AttendancePunchVO vo = attendanceService.punch(dto);
        return success(vo);
    }

    @GetMapping("/statisticsByDay")
    @Operation(summary = "统计按天")
    public CommonResult<Map<String, List<AttendancePunchRecordDO>>> statisticsByDay(@ModelAttribute AttendanceStatisticsByDayDTO dto) {
        Map<String, List<AttendancePunchRecordDO>> vo = attendanceService.statisticsByDay(dto);
        return success(vo);
    }

    @GetMapping("/statisticsByCycle")
    @Operation(summary = "统计按周期")
    public CommonResult<AttendanceStatisticsVO> statisticsByCycle(@ModelAttribute @Valid AttendanceStatisticsByCycleDTO dto) {
        AttendanceStatisticsVO vo = attendanceService.statisticsByCycle(dto);
        return success(vo);
    }
}
