package cn.iocoder.yudao.module.system.job.attendance;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.LocalDateTimeUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.iocoder.yudao.framework.common.Constants;
import cn.iocoder.yudao.framework.common.util.date.DateUtils;
import cn.iocoder.yudao.framework.tenant.core.job.TenantJob;
import cn.iocoder.yudao.module.system.dal.dataobject.attendance.group.AttendanceGroupDO;
import cn.iocoder.yudao.module.system.dal.dataobject.attendance.groupshiftitem.AttendanceGroupShiftItemDO;
import cn.iocoder.yudao.module.system.dal.dataobject.attendance.groupuser.AttendanceGroupUserDO;
import cn.iocoder.yudao.module.system.dal.dataobject.attendance.punchrecord.AttendancePunchRecordDO;
import cn.iocoder.yudao.module.system.dal.mysql.attendance.group.AttendanceGroupMapper;
import cn.iocoder.yudao.module.system.dal.mysql.attendance.groupuser.AttendanceGroupUserMapper;
import cn.iocoder.yudao.module.system.service.attendance.AttendanceService;
import cn.iocoder.yudao.module.system.service.attendance.fixed.AttendanceFixedService;
import cn.iocoder.yudao.module.system.service.attendance.groupshiftitem.AttendanceGroupShiftItemService;
import cn.iocoder.yudao.module.system.service.attendance.punch.dto.AttendanceOnTheDayDTO;
import cn.iocoder.yudao.module.system.service.attendance.punchrecord.AttendancePunchRecordService;
import cn.iocoder.yudao.module.system.service.attendance.scheduling.AttendanceSchedulingService;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.temporal.ChronoUnit;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Component
@Slf4j
public class AttendanceStatisticsJob {

    // TODO: 2024/4/22  - 每天凌晨1点执行 将明天需要考勤的人员统计到数据库中先

    /**
     * / -- 把明天的考勤先入进去 会有什么问题？
     * / -- 1.考勤人员调整？ -- 用户更换考勤组/新人进入考勤组 -
     * （两种 -
     * 1.立即生效 / 删除该用户在前考勤组的明天记录 - 并且新增立即插入当前考勤组规则到表中
     * 2.明天生效 / 不需要处理）
     * / -- 2.调整考勤类型？ -- 调整考勤组考勤类型？（固定/排版）  （两种情况 - 同上解决）
     * / -- 3.调整固定/排版考勤设置  -- （两种情况 - 同上解决）
     * / -- 4.什么时候把未打卡改为缺卡？  定时任务更新 - 未打卡 的并且 时间大于应该打卡时间的
     * / -- 5. 如果我凌晨1点 跑 明天的数据 会有什么问题吗？  好像没什么问题 - 嗯
     */
    @Resource
    private AttendanceService attendanceService;
    @Resource
    private AttendanceGroupMapper attendanceGroupMapper;
    @Resource
    private AttendanceGroupUserMapper attendanceGroupUserMapper;
    @Resource
    private AttendanceFixedService attendanceFixedService;

    @Resource
    private AttendanceSchedulingService attendanceSchedulingService;
    @Resource
    private AttendanceGroupShiftItemService attendanceGroupShiftItemService;
    @Resource
    private AttendancePunchRecordService attendancePunchRecordService;

    @XxlJob("attendanceStatisticsJob")
    @TenantJob // --- ⚠️ 这个注解 会将租户列表拉出来 完了后逐个租户执行 定时任务需要注意
    public ReturnT<String> execute() throws Exception {
        log.info("开始 考勤预设");
        // 获取所有考勤组
        List<AttendanceGroupDO> attendanceGroupDOS = attendanceGroupMapper.selectList();
        // -- 根据考勤组ids 获取所有人员
        List<AttendanceGroupUserDO> attendanceGroupUserDOS = attendanceGroupUserMapper.selectList();
        //将attendanceGroupUserDOS对象中的type相同的userId合并成一个数组 type作为key 用户id列表List<Long>作为value
        Map<Long, List<Long>> groupUserMap = attendanceGroupUserDOS.stream().collect(Collectors.groupingBy(AttendanceGroupUserDO::getAttendanceGroupId,
                Collectors.mapping(AttendanceGroupUserDO::getUserId, Collectors.toList())));
        // -- 获取考勤组下考勤规则 - 将将考勤组分组 - 按类型
        LocalDateTime tomorrowLocalDateTime = LocalDateTimeUtil.offset(LocalDateTime.now(), 1, ChronoUnit.DAYS);
        String time = tomorrowLocalDateTime.format(Constants.REPO_DATE_FORMAT);
        // 获取到考勤组 - 班次 key/value 格式
        Map<Long, Long> map = this.getAttendanceGroupShiftIdGroupByGroup(attendanceGroupDOS, tomorrowLocalDateTime);
        Map<Long, AttendanceGroupDO> groupMap = attendanceGroupDOS.stream().collect(Collectors.toMap(AttendanceGroupDO::getId, v -> v));
        // -- 获取班次子表列表
        if (MapUtil.isNotEmpty(map)) {
            List<AttendanceGroupShiftItemDO> attendanceGroupShiftItemDOList = attendanceGroupShiftItemService.getGroupShiftItemListByShiftIds(new ArrayList<>(map.values()));
            Map<Long, List<AttendanceGroupShiftItemDO>> itemMaps = attendanceGroupShiftItemDOList.stream().collect(Collectors.groupingBy(AttendanceGroupShiftItemDO::getKqAttendanceGroupShiftId));
            List<AttendancePunchRecordDO> attendancePunchRecordDOList = new ArrayList<>();
            for (Map.Entry<Long, Long> entry : map.entrySet()) {
                AttendanceGroupDO attendanceGroupDO = groupMap.get(entry.getKey());
                List<Long> userIds = groupUserMap.get(entry.getKey());
                if (CollectionUtil.isEmpty(userIds)) {
                    continue;
                }
                List<AttendanceGroupShiftItemDO> attendanceGroupShiftItemDOS = itemMaps.get(entry.getValue());
                if (CollectionUtil.isEmpty(attendanceGroupShiftItemDOS)) {
                    continue;
                }
                List<AttendanceOnTheDayDTO> attendanceOnTheDayDTOS = attendanceService.buildAttendanceOnTheDay(attendanceGroupShiftItemDOS);
                for (Long userId : userIds) {
                    for (AttendanceOnTheDayDTO attendanceOnTheDayDTO : attendanceOnTheDayDTOS) {
                        AttendancePunchRecordDO attendancePunchRecordDO = new AttendancePunchRecordDO();
                        attendancePunchRecordDO.setUserId(userId);
                        attendancePunchRecordDO.setAttendanceGroupId(entry.getKey());
                        attendancePunchRecordDO.setAttendanceGroupShiftId(attendanceOnTheDayDTO.getKqAttendanceGroupShiftId());
                        attendancePunchRecordDO.setAttendanceGroupShiftItemId(attendanceOnTheDayDTO.getId());
                        attendancePunchRecordDO.setType(attendanceGroupDO.getType());
                        attendancePunchRecordDO.setPunchType(attendanceGroupDO.getPunchType());
                        attendancePunchRecordDO.setWorkType(attendanceOnTheDayDTO.getType());
                        attendancePunchRecordDO.setLevel(attendanceOnTheDayDTO.getLevel());
                        attendancePunchRecordDO.setStatus(AttendanceOnTheDayDTO.PUNCH_STATUS_UN_PUNCH);
                        attendancePunchRecordDO.setFieldServiceFlag(Constants.FALSE);
                        attendancePunchRecordDO.setNextDayFlag(Constants.TRUE);
                        attendancePunchRecordDO.setDayTime(time);
                        LocalDateTime shouldPunchTime = LocalDateTime.ofInstant(DateUtils.buildHHmmTime(attendanceOnTheDayDTO.getTime()).toInstant(), ZoneId.systemDefault());
                        attendancePunchRecordDO.setShouldPunchTime(shouldPunchTime);
                        attendancePunchRecordDOList.add(attendancePunchRecordDO);
                        // TODO: 2024/5/24
                    }
                }
            }
            // -- 批量
            attendancePunchRecordService.saveBatch(attendancePunchRecordDOList);
        }
        log.info("结束 考勤预设");
        // 返回执行成功
        return ReturnT.SUCCESS;
    }

    private Map<Long, Long> getAttendanceGroupShiftIdGroupByGroup(List<AttendanceGroupDO> attendanceGroupDOS, LocalDateTime localDateTime) {
        Map<Long, Long> map = new HashMap<>();
        List<AttendanceGroupDO> fixedList = attendanceGroupDOS.stream().filter(a -> a.getType().equals(1)).collect(Collectors.toList());
        List<AttendanceGroupDO> schedulingList = attendanceGroupDOS.stream().filter(a -> a.getType().equals(2)).collect(Collectors.toList());
        if (!fixedList.isEmpty()) {
            Map<Long, Long> attendanceFixedMap = attendanceFixedService.getGroupToShiftIdMap(fixedList, localDateTime);
            if (ObjectUtil.isNotEmpty(attendanceFixedMap)) {
                map.putAll(attendanceFixedMap);
            }
        }
        if (!schedulingList.isEmpty()) {
            Map<Long, Long> attendanceSchedulingMap = attendanceSchedulingService.getGroupToShiftIdMap(schedulingList, localDateTime);
            if (ObjectUtil.isNotEmpty(attendanceSchedulingMap)) {
                map.putAll(attendanceSchedulingMap);
            }
        }
        return map;
    }
}
