package cn.iocoder.yudao.module.system.service.attendance.groupshift;

import cn.hutool.core.collection.CollectionUtil;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.module.system.controller.admin.groupshift.vo.AttendanceGroupShiftPageReqVO;
import cn.iocoder.yudao.module.system.controller.admin.groupshift.vo.AttendanceGroupShiftSaveReqVO;
import cn.iocoder.yudao.module.system.controller.admin.groupshift.vo.AttendanceGroupShiftVO;
import cn.iocoder.yudao.module.system.dal.dataobject.attendance.groupshift.AttendanceGroupShiftDO;
import cn.iocoder.yudao.module.system.dal.dataobject.attendance.groupshiftitem.AttendanceGroupShiftItemDO;
import cn.iocoder.yudao.module.system.dal.mysql.attendance.groupshift.AttendanceGroupShiftMapper;
import cn.iocoder.yudao.module.system.dal.mysql.attendance.groupshiftitem.AttendanceGroupShiftItemMapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.system.enums.ErrorCodeConstants.GROUP_SHIFT_NOT_EXISTS;

/**
 * 考勤组班次 Service 实现类
 *
 * @author 艾楷
 */
@Service
@Validated
public class AttendanceGroupShiftServiceImpl implements AttendanceGroupShiftService {

    @Resource
    private AttendanceGroupShiftMapper groupShiftMapper;
    @Resource
    private AttendanceGroupShiftItemMapper groupShiftItemMapper;

    @Override
    public Long createGroupShift(AttendanceGroupShiftSaveReqVO createReqVO) {
        // 插入
        AttendanceGroupShiftDO groupShift = BeanUtils.toBean(createReqVO, AttendanceGroupShiftDO.class);
        groupShiftMapper.insert(groupShift);
        // -- 插入子表
        List<AttendanceGroupShiftItemDO> items = BeanUtils.toBean(createReqVO.getItems(), AttendanceGroupShiftItemDO.class);
        for (AttendanceGroupShiftItemDO item : items) {
            item.setKqAttendanceGroupShiftId(groupShift.getId());
        }
        groupShiftItemMapper.insertBatch(items);
        // 返回
        return groupShift.getId();
    }

    @Override
    public void updateGroupShift(AttendanceGroupShiftSaveReqVO updateReqVO) {
        // 校验存在
        validateGroupShiftExists(updateReqVO.getId());
        // 更新
        AttendanceGroupShiftDO updateObj = BeanUtils.toBean(updateReqVO, AttendanceGroupShiftDO.class);
        // -- 插入子表
        List<AttendanceGroupShiftItemDO> items = BeanUtils.toBean(updateReqVO.getItems(), AttendanceGroupShiftItemDO.class);
        groupShiftItemMapper.updateBatch(items);

        groupShiftMapper.updateById(updateObj);
    }

    @Override
    public void deleteGroupShift(Long id) {
        // 校验存在
        validateGroupShiftExists(id);
        // 删除
        groupShiftMapper.deleteById(id);
    }

    private void validateGroupShiftExists(Long id) {
        if (groupShiftMapper.selectById(id) == null) {
            throw exception(GROUP_SHIFT_NOT_EXISTS);
        }
    }

    @Override
    public AttendanceGroupShiftDO getGroupShift(Long id) {
        return groupShiftMapper.selectById(id);
    }

    @Override
    public PageResult<AttendanceGroupShiftDO> getGroupShiftPage(AttendanceGroupShiftPageReqVO pageReqVO) {
        return groupShiftMapper.selectPage(pageReqVO);
    }

    @Override
    public List<AttendanceGroupShiftVO> allList() {
        List<AttendanceGroupShiftDO> dos = groupShiftMapper.selectList();
        return this.buildShift(dos);
    }

    @Override
    public List<AttendanceGroupShiftVO> listByShiftIds(List<Long> listByShiftIds) {
        List<AttendanceGroupShiftDO> dos = groupShiftMapper.selectBatchIds(listByShiftIds);
        return this.buildShift(dos);
    }

    private List<AttendanceGroupShiftVO> buildShift(List<AttendanceGroupShiftDO> dos) {
        // 拿到ids - 完了后 获取所有的班次子表
        List<Long> ids = dos.stream().map(AttendanceGroupShiftDO::getId).collect(Collectors.toList());
        Map<Long, List<AttendanceGroupShiftItemDO>> map = new HashMap<>();
        if (!ids.isEmpty()) {
            List<AttendanceGroupShiftItemDO> attendanceGroupShiftItemDOS = groupShiftItemMapper.selectList(new LambdaQueryWrapper<AttendanceGroupShiftItemDO>()
                    .in(AttendanceGroupShiftItemDO::getKqAttendanceGroupShiftId, ids));
            map = attendanceGroupShiftItemDOS.stream().collect(Collectors.groupingBy(AttendanceGroupShiftItemDO::getKqAttendanceGroupShiftId));
        }
        List<AttendanceGroupShiftVO> vos = new ArrayList<>();
        for (AttendanceGroupShiftDO item : dos) {
            AttendanceGroupShiftVO attendanceGroupShiftVO = BeanUtils.toBean(item, AttendanceGroupShiftVO.class);
            List<AttendanceGroupShiftItemDO> attendanceGroupShiftItemDOS = map.get(item.getId());
            if (CollectionUtil.isNotEmpty(attendanceGroupShiftItemDOS)) {
                attendanceGroupShiftVO.setAttendanceGroupShiftItemDOList(attendanceGroupShiftItemDOS);
            }
            vos.add(attendanceGroupShiftVO);
        }
        return vos;
    }
}